#include <stdio.h>
#include <math.h>
#include "mytime.h"

// GPU kernel
__global__ void integrate(int *n, double *sum) {
  double h, x;
  int i;
	
  *sum = 0.0;
  h = 1.0 / (double) *n;
  for (i = 1; i <= *n; i++) {
    x = h * ((double)i - 0.5);
    *sum += 4.0 / (1.0 + x*x);
  }
  *sum *= h;
}

int main(int argc, char *argv[]) {
  int n;
  int *n_d; // device copy of n
  double PI25DT = 3.141592653589793238462643;
  double pi;
  double *pi_d; // device copy of pi
  struct timeval startwtime, endwtime, diffwtime;
  
  // Allocate memory on GPU
  cudaMalloc( (void **) &n_d, sizeof(int) * 1 );
  cudaMalloc( (void **) &pi_d, sizeof(double) * 1 );

  while (1) {
    printf("Enter the number of intervals: (0 quits) ");fflush(stdout);
    scanf("%d",&n);

    gettimeofday(&startwtime, NULL);
    if (n == 0)
      break;

    // copy from CPU to GPU
    cudaMemcpy( n_d, &n, sizeof(int) * 1, cudaMemcpyHostToDevice );

    integrate<<< 1, 1 >>>(n_d, pi_d);
    
    // copy back from GPU to CPU
    cudaMemcpy( &pi, pi_d, sizeof(double) * 1, cudaMemcpyDeviceToHost );

    gettimeofday(&endwtime, NULL);
    MINUS_UTIME(diffwtime, endwtime, startwtime);
    printf("pi is approximately %.16f, Error is %.16f\n",
	   pi, fabs(pi - PI25DT));
    printf("wall clock time = %d.%06d\n",
	   diffwtime.tv_sec, diffwtime.tv_usec);
  }

  // free GPU memory
  cudaFree(n_d);
  cudaFree(pi_d);

  return 0;
}
